package com.tzld.piaoquan.recommend.feature.client;

import com.tzld.piaoquan.recommend.feature.model.feature.*;
import lombok.extern.slf4j.Slf4j;
import net.devh.boot.grpc.client.inject.GrpcClient;
import org.springframework.stereotype.Component;

import java.util.Collections;
import java.util.List;

/**
 * @author dyp
 */
@Component
@Slf4j
public class FeatureClient {
    @GrpcClient("recommend-feature")
    private FeatureServiceGrpc.FeatureServiceBlockingStub client;

    public UserFeatureProto getUserFeature(String mid) {
        GetUserFeatureRequest request = GetUserFeatureRequest.newBuilder()
                .setMid(mid)
                .build();
        GetUserFeatureResponse response = client.getUserFeature(request);
        if (response == null || !response.hasResult()) {
            log.info("getUserFeature grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getUserFeature grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getUserFeature();
    }

    public VideoFeatureProto getVideoFeature(Long videoId) {
        GetVideoFeatureRequest request = GetVideoFeatureRequest.newBuilder()
                .setVideoId(String.valueOf(videoId))
                .build();
        GetVideoFeatureResponse response = client.getVideoFeature(request);
        if (response == null || !response.hasResult()) {
            log.info("getVideoFeature grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getVideoFeature grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.hasVideoFeature() ? response.getVideoFeature() : null;
    }

    public List<VideoFeatureProto> getAllVideoFeature(List<Long> videoIds) {
        GetAllVideoFeatureRequest.Builder buildr = GetAllVideoFeatureRequest.newBuilder();
        for (Long videoId : videoIds) {
            buildr.addVideoId(videoId + "");
        }
        GetAllVideoFeatureResponse response = client.getAllVideoFeature(buildr.build());
        if (response == null || !response.hasResult()) {
            log.info("getAllVideoFeature grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getAllVideoFeature grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getVideoFeatureCount() > 0
                ? response.getVideoFeatureList()
                : Collections.emptyList();
    }

    public UserAdFeatureProto getUserAdFeature(String mid) {
        GetUserAdFeatureRequest request = GetUserAdFeatureRequest .newBuilder()
                .setMid(mid)
                .build();
        GetUserAdFeatureResponse response = client.getUserAdFeature(request);
        if (response == null || !response.hasResult()) {
            log.info("getUserAdFeature grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getUserAdFeature grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getUserAdFeature();
    }

    public AdItemFeatureProto getAdItemFeature(String adId) {
        GetAdItemFeatureRequest request = GetAdItemFeatureRequest .newBuilder()
                .setAdId(adId)
                .build();
        GetAdItemFeatureResponse response = client.getAdItemFeature(request);
        if (response == null || !response.hasResult()) {
            log.info("getAdItemFeature grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getAdItemFeature grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getAdItemFeature();
    }

    public List<AdItemFeatureProto> getAllAdItemFeature(List<String> adIdList) {
        GetAllAdItemFeatureRequest.Builder buildr = GetAllAdItemFeatureRequest.newBuilder();
        buildr.addAllAdId(adIdList);
        GetAllAdItemFeatureResponse response = client.getAllAdItemFeature(buildr.build());
        if (response == null || !response.hasResult()) {
            log.info("getALLAdItemFeature grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getALLAdItemFeature grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getAdItemFeatureCount() > 0
                ? response.getAdItemFeatureList()
                : Collections.emptyList();
    }

    public List<GetUserFeatureInfo> getUserFeatureInfo(List<String> mids) {
        GetUserFeatureInfoRequest request = GetUserFeatureInfoRequest.newBuilder()
                .addAllMids(mids)
                .build();
        GetUserFeatureInfoResponse response = client.getUserFeatureInfo(request);
        if (response == null || !response.hasResult()) {
            log.info("getUserFeatureInfo grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getUserFeatureInfo grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getUserFeatureInfoCount() > 0
                ? response.getUserFeatureInfoList()
                : Collections.emptyList();
    }

    public List<GetVideoFeatureInfo> getVideoFeatureInfo(List<Long> videoIds) {
        GetVideoFeatureInfoRequest request = GetVideoFeatureInfoRequest.newBuilder()
                .addAllVideoIds(videoIds)
                .build();
        GetVideoFeatureInfoResponse response = client.getVideoFeatureInfo(request);
        if (response == null || !response.hasResult()) {
            log.info("getVideoFeatureInfo grpc error");
            return null;
        }
        if (response.getResult().getCode() != 1) {
            log.info("getVideoFeatureInfo grpc code={}, msg={}", response.getResult().getCode(),
                    response.getResult().getMessage());
            return null;
        }
        return response.getVideoFeatureInfoList().size() > 0
                ? response.getVideoFeatureInfoList()
                : Collections.emptyList();
    }

}
