package com.tzld.piaoquan.recommend.feature.domain.ad.base;

import lombok.Data;

import java.util.HashMap;
import java.util.Map;

@Data
public class AdItemBytesFeature {

    private final byte[] adId;

    private final byte[] advertiserId;

    private final byte[] campaignId;

    private final byte[] creative;

    // adid 维度，也就是planid
    private Map<String, byte[]> day1_cnt_features;
    // 3天内统计量
    private Map<String, byte[]> day3_cnt_features;
    // 7天内统计量
    private Map<String, byte[]> day7_cnt_features;
    // 3个月统计量
    private Map<String, byte[]> month3_cnt_features;


    // creative 维度
    private Map<String, byte[]> creative_day1_cnt_features;
    // 3天内统计量
    private Map<String, byte[]> creative_day3_cnt_features;
    // 7天内统计量
    private Map<String, byte[]> creative_day7_cnt_features;
    // 3个月统计量
    private Map<String, byte[]> creative_month3_cnt_features;


    // advertiserid 维度
    private Map<String, byte[]> advertiser_day1_cnt_features;
    // 3天内统计量
    private Map<String, byte[]> advertiser_day3_cnt_features;
    // 7天内统计量
    private Map<String, byte[]> advertiser_day7_cnt_features;
    // 3个月统计量
    private Map<String, byte[]> advertiser_month3_cnt_features;


    private static void addAdActionFeature(Map<String, byte[]>  dayCntMap, AdActionFeature adActionFeature ){

        dayCntMap.put("view", String.valueOf(ceilLog(adActionFeature.getAdView())).getBytes());
        dayCntMap.put("click", String.valueOf(ceilLog(adActionFeature.getAdClick())).getBytes());
        dayCntMap.put("conversion", String.valueOf(ceilLog(adActionFeature.getAdConversion())).getBytes());
        dayCntMap.put("ctr", String.valueOf(bucketRatioFeature(adActionFeature.getCtr())).getBytes());
        dayCntMap.put("cvr", String.valueOf(bucketRatioFeature(adActionFeature.getCvr())).getBytes());
    }

    private static double ceilLog(Double key) {
        return Math.ceil(Math.log(key + 1));
    }

    private static double bucketRatioFeature(Double key) {
        long bucket = Math.round(Math.log((key + 1) * 50));
        if (bucket > 50)
            bucket = 50;
        return (double) bucket;
    }



    public AdItemBytesFeature(AdItemFeature feature) {

        adId = feature.getAdId().getBytes();
        advertiserId = feature.getAdvertiserId().getBytes();
        campaignId = feature.getCampaignId().getBytes();
        creative = feature.getCreativeId().getBytes();


        // adid 维度  planid
        // 1 day statistic
        day1_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(day1_cnt_features, feature.getDay1_cnt_features());

        // 3 day statistic
        day3_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(day3_cnt_features, feature.getDay3_cnt_features());

        // 7 day statistic
        day7_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(day7_cnt_features, feature.getDay7_cnt_features());

        // 3 month statisic
        month3_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(month3_cnt_features, feature.getMonth3_cnt_features());


        // creative 维度  creativeid
        // 1 day statistic
        creative_day1_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(creative_day1_cnt_features, feature.getCreative_1day_cnt_features());

        // 3 day statistic
        creative_day3_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(creative_day3_cnt_features, feature.getCreative_3day_cnt_features());


        // 7 day statistic
        creative_day7_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(creative_day7_cnt_features, feature.getCreative_7day_cnt_features());


        // 3 month statisic
        creative_month3_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(creative_month3_cnt_features, feature.getCreative_3month_cnt_features());


        // advertiser 维度  planid
        // 1 day statistic
        advertiser_day1_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(advertiser_day1_cnt_features, feature.getAdvertiser_1day_cnt_features());


        // 3 day statistic
        advertiser_day3_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(advertiser_day3_cnt_features, feature.getAdvertiser_3day_cnt_features());


        // 7 day statistic
        advertiser_day7_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(advertiser_day7_cnt_features, feature.getAdvertiser_7day_cnt_features());

        // 3 month statisic
        advertiser_month3_cnt_features = new HashMap<String, byte[]>();
        addAdActionFeature(advertiser_month3_cnt_features, feature.getAdvertiser_3month_cnt_features());



    }

}
