package com.tzld.piaoquan.recommend.feature.domain.ad.feature;

import com.google.common.collect.ArrayListMultimap;
import com.google.common.collect.ListMultimap;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdItemBytesFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdRequestContextBytesFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.UserAdBytesFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.enums.VlogAdFeatureGroup;
import com.tzld.piaoquan.recommend.feature.model.sample.BaseFeature;
import com.tzld.piaoquan.recommend.feature.model.sample.FeatureGroup;
import com.tzld.piaoquan.recommend.feature.model.sample.LRSamples;

import java.util.List;


public abstract class LRBytesFeatureExtractorBase {
    private static final double DEFAULT_USER_CTR_GROUP = 10.0;
    private static final double DEFAULT_ARTICLE_CTR_GROUP = 100.0;


    private BytesUtils utils;
    //Feature Group & Features
    ListMultimap<FeatureGroup, BaseFeature> features = ArrayListMultimap.create();
    int groupCount;


    LRBytesFeatureExtractorBase() {
        groupCount = VlogAdFeatureGroup.values().length;
        BytesGroup[] groups = new BytesGroup[groupCount];
        for (VlogAdFeatureGroup g: VlogAdFeatureGroup.values()) {
            groups[g.ordinal()] = new BytesGroup(g.ordinal(),
                    g.getGroupName(), g.getGroupNameBytes());
        }
        utils = new BytesUtils(groups);
    }

    private FeatureGroup makeGroup(VlogAdFeatureGroup group){
        FeatureGroup.Builder g = FeatureGroup.newBuilder();
        g.setType("1");
        g.setName(group.getGroupName());
        g.setId(group.ordinal());
        return g.build();
    };


    void makeFea(VlogAdFeatureGroup group, byte[] value) {
        FeatureGroup featureGroup = makeGroup(group);
        BaseFeature feature = utils.makeFea(group.ordinal(), value);
        features.put(featureGroup, feature);
    }

    void makeFea(VlogAdFeatureGroup group, byte[][] list) {
        FeatureGroup g = makeGroup(group);
        List<BaseFeature> featureList = utils.makeFea(group.ordinal(), list);
        features.putAll(g, featureList);
    }

    // 增加交叉特征的方法
    void makeFea(VlogAdFeatureGroup group, final byte[] left, final byte[] right) {
        FeatureGroup g = makeGroup(group);
        BaseFeature feature = utils.makeFea(group.ordinal(), left, right);
        features.put(g, feature);
    }



    public ListMultimap<FeatureGroup, BaseFeature> getFeatures() {
        return features;
    }

    public abstract LRSamples single(UserAdBytesFeature userBytesFeature,
                                     AdItemBytesFeature videoBytesFeature,
                                     AdRequestContextBytesFeature requestContextBytesFeature);


}
