package com.tzld.piaoquan.recommend.feature.produce;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.produce.service.ODPSService;
import com.tzld.piaoquan.recommend.feature.produce.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang.StringUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;
import org.apache.spark.api.java.function.Function;

import java.util.*;

/**
 * @author dyp
 */
@Slf4j
public class FeatureDiff {

    private static ODPSService odpsService = new ODPSService();

    private static Set<String> abCodes;

    static {
        abCodes = new HashSet<>();
        abCodes.add("ab0");
        abCodes.add("ab1");
        abCodes.add("ab2");
        abCodes.add("ab3");
//        tableToCol = new HashMap<>();
//
//        tableToCol.put("alg_mid_feature_play", "c1_feature");
//        tableToCol.put("alg_mid_feature_share_and_return", "c2_feature");
//        tableToCol.put("alg_mid_feature_play_tags", "c3_feature");
//        tableToCol.put("alg_mid_feature_return_tags", "c4_feature");
//        tableToCol.put("alg_mid_feature_share_tags", "c5_feature");
//        tableToCol.put("alg_mid_feature_feed_exp_share_tags", "c6_feature");
//        tableToCol.put("alg_mid_feature_feed_exp_return_tags", "c7_feature");
//        tableToCol.put("alg_mid_feature_sharecf", "c8_feature");
//        tableToCol.put("alg_mid_feature_returncf", "c9_feature");
//        tableToCol.put("alg_vid_feature_all_exp", "b1_feature");
//        tableToCol.put("alg_vid_feature_all_share", "b2_feature");
//        tableToCol.put("alg_vid_feature_all_return", "b3_feature");
////        tableToCol.put("alg_vid_feature_head_play", "b4_feature");
////        tableToCol.put("alg_vid_feature_feed_play", "b5_feature");
//        tableToCol.put("alg_vid_feature_exp2share", "b6_feature");
//        tableToCol.put("alg_vid_feature_share2return", "b7_feature");
//        tableToCol.put("alg_vid_feature_feed_noflow_exp", "b8_feature");
//        tableToCol.put("alg_vid_feature_feed_noflow_root_share", "b9_feature");
//        tableToCol.put("alg_vid_feature_feed_noflow_root_return", "b10_feature");
//        tableToCol.put("alg_vid_feature_feed_flow_exp", "b11_feature");
//        tableToCol.put("alg_vid_feature_feed_flow_root_share", "b12_feature");
//        tableToCol.put("alg_vid_feature_feed_flow_root_return", "b13_feature");
////        tableToCol.put("alg_vid_feature_feed_apptype_exp", "b14_feature");
////        tableToCol.put("alg_vid_feature_feed_apptype_root_share", "b15_feature");
////        tableToCol.put("alg_vid_feature_feed_apptype_root_return", "b16_feature");
//        tableToCol.put("alg_vid_feature_feed_province_exp", "b17_feature");
//        tableToCol.put("alg_vid_feature_feed_province_root_share", "b18_feature");
//        tableToCol.put("alg_vid_feature_feed_province_root_return", "b19_feature");
    }

    private static void addUser(Map<String, String> tableToCol) {
        tableToCol.put("alg_mid_feature_play", "c1_feature");
        tableToCol.put("alg_mid_feature_share_and_return", "c2_feature");
        tableToCol.put("alg_mid_feature_play_tags", "c3_feature");
        tableToCol.put("alg_mid_feature_return_tags", "c4_feature");
        tableToCol.put("alg_mid_feature_share_tags", "c5_feature");
        tableToCol.put("alg_mid_feature_feed_exp_share_tags", "c6_feature");
        tableToCol.put("alg_mid_feature_feed_exp_return_tags", "c7_feature");
        tableToCol.put("alg_mid_feature_sharecf", "c8_feature");
        tableToCol.put("alg_mid_feature_returncf", "c9_feature");
    }

    private static void addVideo(Map<String, String> tableToCol) {
        tableToCol.put("alg_vid_feature_all_exp", "b1_feature");
        tableToCol.put("alg_vid_feature_all_share", "b2_feature");
        tableToCol.put("alg_vid_feature_all_return", "b3_feature");
//        tableToCol.put("alg_vid_feature_head_play", "b4_feature");
//        tableToCol.put("alg_vid_feature_feed_play", "b5_feature");
        tableToCol.put("alg_vid_feature_exp2share", "b6_feature");
        tableToCol.put("alg_vid_feature_share2return", "b7_feature");
        tableToCol.put("alg_vid_feature_feed_noflow_exp", "b8_feature");
        tableToCol.put("alg_vid_feature_feed_noflow_root_share", "b9_feature");
        tableToCol.put("alg_vid_feature_feed_noflow_root_return", "b10_feature");
        tableToCol.put("alg_vid_feature_feed_flow_exp", "b11_feature");
        tableToCol.put("alg_vid_feature_feed_flow_root_share", "b12_feature");
        tableToCol.put("alg_vid_feature_feed_flow_root_return", "b13_feature");
//        tableToCol.put("alg_vid_feature_feed_apptype_exp", "b14_feature");
//        tableToCol.put("alg_vid_feature_feed_apptype_root_share", "b15_feature");
//        tableToCol.put("alg_vid_feature_feed_apptype_root_return", "b16_feature");
        tableToCol.put("alg_vid_feature_feed_province_exp", "b17_feature");
        tableToCol.put("alg_vid_feature_feed_province_root_share", "b18_feature");
        tableToCol.put("alg_vid_feature_feed_province_root_return", "b19_feature");
    }

    public static void main(String[] args) {


        Map<String, String> tableToCol = new HashMap<>();
        if (args[3].equals("0")) {
            addVideo(tableToCol);
        } else if (args[3].equals("1")) {
            addUser(tableToCol);
        } else {
            addVideo(tableToCol);
            addUser(tableToCol);
        }


        SparkConf sparkConf = new SparkConf()
                // .setMaster("local")
                .setAppName("odps sync to redis");
        JavaSparkContext jsc = new JavaSparkContext(sparkConf);

        // ODPS
        log.info("read odps");
        String project = "loghubods";
        String table = "alg_recsys_sample_all_new";
        String partition = "dt=" + args[1] + ",hh=" + args[2];

        int partitionNum = Integer.valueOf(args[0]);
        JavaRDD<Map<String, String>> fieldValues = odpsService.read(jsc, project, table, partition, partitionNum);
        if (fieldValues == null) {
            log.info("odps empty");
            return;
        }

        long diffCount =
                fieldValues.repartition(partitionNum).filter(new Function<Map<String, String>,
                        Boolean>() {
                    @Override
                    public Boolean call(Map<String, String> map) throws Exception {

                        if (StringUtils.isNotBlank(map.get("flowpool"))) {
                            return false;
                        }

                        if (!abCodes.contains(map.get("abcode"))) {
                            return false;
                        }

                        Map<String, Map<String, String>> metaFeatureMap = JSONUtils.fromJson(map.get("metafeaturemap"),
                                new TypeToken<Map<String, Map<String, String>>>() {
                                }, Collections.emptyMap());
                        for (Map.Entry<String, String> e : tableToCol.entrySet()) {
                            Map<String, String> offline = JSONUtils.fromJson(map.get(e.getValue()), new TypeToken<Map<String, String>>() {
                            }, Collections.emptyMap());

                            Map<String, String> online = metaFeatureMap.getOrDefault(e.getKey(), new HashMap<>());


                            if (offline.size() != online.size()) {
                                return true;
                            }

                            for (Map.Entry<String, String> offlineE : offline.entrySet()) {
                                if (!StringUtils.equals(online.get(offlineE.getKey()), offlineE.getValue())) {
                                    return true;
                                }
                            }
                        }
                        return false;
                    }
                }).count();

        log.info("diff count {}", diffCount);

    }

}
