package com.tzld.piaoquan.recommend.feature.produce;

import com.tzld.piaoquan.recommend.feature.produce.model.DTSConfig;
import com.tzld.piaoquan.recommend.feature.produce.service.CMDService;
import com.tzld.piaoquan.recommend.feature.produce.service.DTSConfigService;
import com.tzld.piaoquan.recommend.feature.produce.service.ODPSService;
import com.tzld.piaoquan.recommend.feature.produce.service.RedisService;
import com.tzld.piaoquan.recommend.feature.produce.util.JSONUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.math.NumberUtils;
import org.apache.spark.SparkConf;
import org.apache.spark.api.java.JavaRDD;
import org.apache.spark.api.java.JavaSparkContext;

import java.util.Map;

/**
 * @author dyp
 */
@Slf4j
public class ODPSToRedis {
    private static CMDService cmdService = new CMDService();
    private static ODPSService odpsService = new ODPSService();

    public static void main(String[] args) {

        log.info("args {}", JSONUtils.toJson(args));

        Map<String, String> argMap = cmdService.parse(args);

//        argMap.put("project", "loghubods");
//        argMap.put("table", "alg_mid_feature_share_and_return");
//        argMap.put("dt", "20240905");
//        argMap.put("hh", "13");
//        argMap.put("mi", "00");
//        argMap.put("env", "test");
//        argMap.put("odpsBatchSize", "300000");

        if (MapUtils.isEmpty(argMap)) {
            log.error("args is empty");
            return;
        }

        SparkConf sparkConf = new SparkConf()
                //.setMaster("local")
                .setAppName("odps sync to redis : " + argMap.get("table"));
        for (Map.Entry<String, String> e : argMap.entrySet()) {
            sparkConf.set(e.getKey(), e.getValue());
        }
        JavaSparkContext jsc = new JavaSparkContext(sparkConf);

        // ODPS
        log.info("read odps");
        String env = argMap.get("env");
        DTSConfigService dtsConfigService = new DTSConfigService(env);
        DTSConfig config = dtsConfigService.getDTSConfig(argMap);
        if (config == null || !config.selfCheck()) {
            log.error("dts config error");
            return;
        }

        long count = 0;
        int retry = 50;
        while (count <= 0 && retry-- > 0) {
            count = odpsService.count(config, argMap);
            if (count <= 0) {
                if (retry <= 0) {
                    // TODO 报警
                    Map<String, String> partitionMap = odpsService.getLastestPartition(config, argMap);
                    argMap.putAll(partitionMap);
                    count = odpsService.count(config, argMap);
                    if (count <= 0) {
                        return;
                    }
                } else {
                    try {
                        log.info("sleep {}", retry);
                        Thread.sleep(60000);
                    } catch (InterruptedException e) {
                        e.printStackTrace();
                    }
                }
            }
        }

        log.info("odps count {}", count);
        long odpsBatchSize = NumberUtils.toLong(argMap.getOrDefault("odpsBatchSize", "200000"));
        argMap.put("partitionNum", String.valueOf(count / odpsBatchSize + 1));

        JavaRDD<Map<String, String>> fieldValues = odpsService.read(jsc, config, argMap);
        if (fieldValues == null) {
            log.info("odps empty");
            return;
        }
        //log.info("odps count {}", fieldValues.count());

        // RDD
        log.info("sync redis");
        RedisService redisService = new RedisService(env);
        int partitionNum = NumberUtils.toInt(argMap.get("partitionNum"), 10);
        fieldValues.repartition(partitionNum).foreachPartition(iterator -> {
            redisService.mSetV2(iterator, config);
        });
        jsc.stop();
    }

}
