package com.tzld.piaoquan.recommend.feature.produce.service;

import com.tzld.piaoquan.recommend.feature.produce.model.DTSConfig;
import com.tzld.piaoquan.recommend.feature.produce.util.JSONUtils;
import org.apache.commons.collections.CollectionUtils;
import org.apache.commons.collections.MapUtils;
import org.apache.commons.lang3.StringUtils;
import org.apache.commons.lang3.tuple.Pair;
import redis.clients.jedis.Jedis;
import redis.clients.jedis.Pipeline;

import java.io.Serializable;
import java.util.HashMap;
import java.util.Iterator;
import java.util.List;
import java.util.Map;
import java.util.concurrent.TimeUnit;

/**
 * https://help.aliyun.com/zh/maxcompute/user-guide/java-sdk-1/?spm=a2c4g.11174283.0.0.6d0111c1E15lI3
 *
 * @author dyp
 */
public class RedisService implements Serializable {
    private int port = 6379;
    private String password = "";
    private String hostName = "";

    public RedisService(String env) {
        if (StringUtils.equals(env, "prod")) {
            password = "Wqsd@2019";
            hostName = "r-bp1pi8wyv6lzvgjy5z.redis.rds.aliyuncs.com";
        } else {
            password = "Wqsd@2019";
            hostName = "r-bp1wwqqkjqwwkxgbup.redis.rds.aliyuncs.com";
        }
    }

    private void mSetEx(Iterator<Pair<String, String>> keyValues, long expire, TimeUnit timeUnit) {
        Jedis jedis = new Jedis(hostName, port);
        jedis.auth(password);
        Pipeline pipeline = jedis.pipelined();
        while (keyValues.hasNext()) {
            Pair<String, String> p = keyValues.next();
            pipeline.setex(p.getLeft(), timeUnit.toSeconds(expire), p.getRight());
        }
        pipeline.sync();
        jedis.close();
    }

    public void mSet(Iterator<Pair<String, String>> keyValues, DTSConfig config) {
        long expire = config.getRedis().getExpire();
        mSetEx(keyValues, expire, TimeUnit.SECONDS);
    }


    public void mSetV2(Iterator<Map<String, String>> dataIte, DTSConfig config) {
        Map<String, String> batch = new HashMap<>();
        Jedis jedis = new Jedis(hostName, port);
        jedis.auth(password);
        long expire = config.getRedis().getExpire();
        while (dataIte.hasNext()) {
            Map<String, String> record = dataIte.next();
            String redisKey = redisKey(record, config);
            String value = JSONUtils.toJson(record);
            batch.put(redisKey, value);
            if (batch.size() >= 5000) {
                mSet(jedis, batch, expire, TimeUnit.SECONDS);
                batch.clear();
            }
        }
        if (MapUtils.isNotEmpty(batch)) {
            mSet(jedis, batch, expire, TimeUnit.SECONDS);
        }
        jedis.close();
    }

    private void mSet(Jedis jedis, Map<String, String> batch, long expire, TimeUnit timeUnit) {
        long expireSeconds = timeUnit.toSeconds(expire);

        Pipeline pipeline = jedis.pipelined();
        for (Map.Entry<String, String> e : batch.entrySet()) {
            pipeline.setex(e.getKey(), expireSeconds, e.getValue());
        }
        pipeline.sync();

    }

    public String redisKey(Map<String, String> fieldValueMap, DTSConfig config) {
        // Note：写入和读取的key生成规则应保持一致
        List<String> fields = config.getRedis().getKey();
        if (CollectionUtils.isEmpty(fields)) {
            return config.getRedis().getPrefix();
        }
        StringBuilder sb = new StringBuilder();
        if (StringUtils.isNotBlank(config.getRedis().getPrefix())) {
            sb.append(config.getRedis().getPrefix());
        }
        for (String field : fields) {
            sb.append(":");
            sb.append(fieldValueMap.get(field));
        }
        return sb.toString();
    }
}