package com.tzld.piaoquan.recommend.feature.config;

import org.apache.commons.pool2.impl.GenericObjectPoolConfig;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.boot.context.properties.ConfigurationProperties;
import org.springframework.context.annotation.Bean;
import org.springframework.context.annotation.Configuration;
import org.springframework.context.annotation.Primary;
import org.springframework.data.redis.connection.RedisConnectionFactory;
import org.springframework.data.redis.connection.RedisStandaloneConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceClientConfiguration;
import org.springframework.data.redis.connection.lettuce.LettuceConnectionFactory;
import org.springframework.data.redis.connection.lettuce.LettucePoolingClientConfiguration;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.data.redis.serializer.StringRedisSerializer;

@Configuration
public class TairTemplateConfig {

    @Bean("tairPool")
    @ConfigurationProperties(prefix = "spring.tair.lettuce.pool")
    public GenericObjectPoolConfig<LettucePoolingClientConfiguration> tairPool() {
        return new GenericObjectPoolConfig<>();
    }

    @Bean("tairConfig")
    @ConfigurationProperties(prefix = "spring.tair")
    public RedisStandaloneConfiguration tairConfig() {
        return new RedisStandaloneConfiguration();
    }

    @Bean("tairFactory")
    @Primary
    public LettuceConnectionFactory factory(@Qualifier("tairPool") GenericObjectPoolConfig<LettucePoolingClientConfiguration> tairPool,
                                            @Qualifier("tairConfig") RedisStandaloneConfiguration tairConfig) {
        LettuceClientConfiguration lettuceClientConfiguration =
                LettucePoolingClientConfiguration.builder().poolConfig(tairPool).build();
        return new LettuceConnectionFactory(tairConfig, lettuceClientConfiguration);
    }

    @Bean(name = "tairTemplate")
    public RedisTemplate<String, String> getTairTemplate(@Qualifier("tairFactory") RedisConnectionFactory factory) {
        return buildTairTemplateByString(factory);
    }

    private RedisTemplate<String, String> buildTairTemplateByString(RedisConnectionFactory factory) {
        RedisTemplate<String, String> redisTemplate = new RedisTemplate<>();
        redisTemplate.setConnectionFactory(factory);
        // key的序列化类型 保证可读性
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        redisTemplate.setValueSerializer(new StringRedisSerializer());
        redisTemplate.setHashKeySerializer(new StringRedisSerializer());
        redisTemplate.setHashValueSerializer(new StringRedisSerializer());
        return redisTemplate;
    }


    @Bean(name = "byteTairTemplate")
    public RedisTemplate<String, byte[]> byteRedisTemplate(@Qualifier("tairFactory") RedisConnectionFactory factory) {
        RedisTemplate<String, byte[]> redisTemplate = new RedisTemplate<>();
        redisTemplate.setConnectionFactory(factory);
        // key的序列化类型 保证可读性
        redisTemplate.setKeySerializer(new StringRedisSerializer());
        redisTemplate.setValueSerializer(new ByteArrayRedisSerializer());
        redisTemplate.setHashKeySerializer(new StringRedisSerializer());
        redisTemplate.setHashValueSerializer(new ByteArrayRedisSerializer());
        return redisTemplate;
    }

}
