package com.tzld.piaoquan.recommend.feature.grpcservice;

import com.tzld.piaoquan.recommend.feature.client.ProtobufUtils;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.*;
import com.tzld.piaoquan.recommend.feature.service.*;
import io.grpc.stub.StreamObserver;
import lombok.extern.slf4j.Slf4j;
import net.devh.boot.grpc.server.service.GrpcService;
import org.springframework.beans.factory.annotation.Autowired;

import java.util.List;

/**
 * @author dyp
 */
@GrpcService
@Slf4j
public class FeatureGrpcService extends FeatureServiceGrpc.FeatureServiceImplBase {
    @Autowired
    private UserFeatureService userFeatureService;
    @Autowired
    private VideoFeatureService videoFeatureService;

    @Autowired
    private AdItemFeatureService adItemFeatureService;

    @Autowired
    private UserAdFeatureService userAdFeatureService;

    @Autowired
    private UserAndVideoFeatureService userAndVideoFeatureService;

    @Override
    public void getUserFeature(GetUserFeatureRequest request, StreamObserver<GetUserFeatureResponse> responseObserver) {
        log.info("FeatureGrpcService queryUserFeature request={}", ProtobufUtils.toJson(request));
        GetUserFeatureResponse response = userFeatureService.getUserFeature(request);
        log.info("FeatureGrpcService queryUserFeature response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }

    @Override
    public void getVideoFeature(GetVideoFeatureRequest request,
                                StreamObserver<GetVideoFeatureResponse> responseObserver) {
        log.info("FeatureGrpcService queryVideoFeature request={}", ProtobufUtils.toJson(request));
        GetVideoFeatureResponse response = videoFeatureService.getVideoFeature(request);
        log.info("FeatureGrpcService queryVideoFeature response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }

    @Override
    public void getAllVideoFeature(GetAllVideoFeatureRequest request,
                                   StreamObserver<GetAllVideoFeatureResponse> responseObserver) {
        log.info("FeatureGrpcService getAllVideoFeature request={}", ProtobufUtils.toJson(request));
        GetAllVideoFeatureResponse response = videoFeatureService.getAllVideoFeature(request);
        log.info("FeatureGrpcService getAllVideoFeature response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }

    @Override
    public void getUserAdFeature(GetUserAdFeatureRequest request,
                                   StreamObserver<GetUserAdFeatureResponse> responseObserver) {
        log.info("FeatureGrpcService getUserAdFeature request={}", ProtobufUtils.toJson(request));
        GetUserAdFeatureResponse response = userAdFeatureService.getUserAdFeature(request);
        log.info("FeatureGrpcService getUserAdFeature response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }

    @Override
    public void getAdItemFeature(GetAdItemFeatureRequest request,
                                 StreamObserver<GetAdItemFeatureResponse> responseObserver) {
        log.info("FeatureGrpcService getAdItemFeatureRequest request={}", ProtobufUtils.toJson(request));
        GetAdItemFeatureResponse response = adItemFeatureService.getAdItemFeature(request);
        log.info("FeatureGrpcService getAdItemFeatureRequest response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }

    @Override
    public void getAllAdItemFeature(GetAllAdItemFeatureRequest request,
                                 StreamObserver<GetAllAdItemFeatureResponse> responseObserver) {
        log.info("FeatureGrpcService getAllAdItemFeature request={}", ProtobufUtils.toJson(request));
        GetAllAdItemFeatureResponse response = adItemFeatureService.getAllAdItemFeature(request);
        log.info("FeatureGrpcService getAllAdItemFeature response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }

    @Override
    public void getUserFeatureInfo(GetUserFeatureInfoRequest request, StreamObserver<GetUserFeatureInfoResponse> responseObserver) {
        log.info("FeatureGrpcService getUserFeatureInfo request={}", ProtobufUtils.toJson(request));
        List<GetUserFeatureInfo> getUserFeatureInfoList = userAndVideoFeatureService.queryUserFeature(request.getMidsList());
        GetUserFeatureInfoResponse response = GetUserFeatureInfoResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1))
                .addAllUserFeatureInfo(getUserFeatureInfoList)
                .build();
        log.info("FeatureGrpcService getUserFeatureInfo response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();

    }

    @Override
    public void getVideoFeatureInfo(GetVideoFeatureInfoRequest request, StreamObserver<GetVideoFeatureInfoResponse> responseObserver) {
        log.info("FeatureGrpcService getVideoFeatureInfo request={}", ProtobufUtils.toJson(request));
        List<GetVideoFeatureInfo> getVideoFeatureInfoList = userAndVideoFeatureService.queryVideoFeature(request.getVideoIdsList());
        GetVideoFeatureInfoResponse response = GetVideoFeatureInfoResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1))
                .addAllVideoFeatureInfo(getVideoFeatureInfoList)
                .build();
        log.info("FeatureGrpcService getVideoFeatureInfo response={}", ProtobufUtils.toJson(response));
        responseObserver.onNext(response);
        responseObserver.onCompleted();
    }
}
