package com.tzld.piaoquan.recommend.feature.service;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdItemFeature;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.*;
import com.tzld.piaoquan.recommend.feature.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

@Service
@Slf4j
public class AdItemFeatureService  extends AbstractFeatureService<String, AdItemFeature> {

    public AdItemFeatureService() {
        super.emptyData = "{}";
        super.emptyDataExpire = 0;
        super.defaultExpire = 0;
        super.keyFunc = k -> String.format("ad:%s", k);
        super.typeToken = new TypeToken<AdItemFeature>() {
        };

        super.defaultValueFunc = k -> {
            AdItemFeature feature = new AdItemFeature();
            feature.setAdId(k);
            return feature;
        };

        int maximumSize = 10000;
        int refreshAfterWrite = 60;
        int expireAfterWrite = 60;
        int expireAfterAccess = 60;
        initLocalCache(maximumSize, refreshAfterWrite, expireAfterWrite, expireAfterAccess);
    }

    public GetAdItemFeatureResponse getAdItemFeature(GetAdItemFeatureRequest request) {
        String adId = request.getAdId();
        if (StringUtils.isBlank(adId)) {
            return GetAdItemFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        AdItemFeature adItemFeature = super.get(adId);
        GetAdItemFeatureResponse.Builder builder = GetAdItemFeatureResponse.newBuilder();
        builder.setResult(Result.newBuilder().setCode(1));
        if (adItemFeature != null) {
            builder.setAdItemFeature(FeatureConverter.convertToProto(adItemFeature));
        }
        return builder.build();
    }

    public GetAllAdItemFeatureResponse getAllAdItemFeature(GetAllAdItemFeatureRequest request) {
        if (request.getAdIdCount() == 0) {
            return GetAllAdItemFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        List<String> adIdList = request.getAdIdList();

        Map<String, AdItemFeature> itemFeatureMap = super.getAll(adIdList);
        if (MapUtils.isEmpty(itemFeatureMap)) {
            return GetAllAdItemFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }

        return GetAllAdItemFeatureResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1))
                .addAllAdItemFeature(CommonCollectionUtils.toList(itemFeatureMap.values(), FeatureConverter::convertToProto))
                .build();
    }
}
