package com.tzld.piaoquan.recommend.feature.service;

import com.google.common.base.Strings;
import com.tzld.piaoquan.recommend.feature.domain.video.base.ItemFeature;
import com.tzld.piaoquan.recommend.feature.domain.video.base.UserActionFeature;
import com.tzld.piaoquan.recommend.feature.domain.video.base.UserFeature;

import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdActionFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdItemFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.UserAdFeature;

import com.tzld.piaoquan.recommend.feature.model.feature.*;

/**
 * @author dyp
 */
public class FeatureConverter {
    public static UserFeatureProto convert(UserFeature userFeature) {
        if (userFeature == null) {
            return null;
        }
        UserFeatureProto.Builder builder = UserFeatureProto.newBuilder();
        builder.setUid(Strings.nullToEmpty(userFeature.getUid()))
                .setMid(Strings.nullToEmpty(userFeature.getMid()))
                .setUserCycleBucket7Day(Strings.nullToEmpty(userFeature.getUser_cycle_bucket_7days()))
                .setUserCycleBucket30Day(Strings.nullToEmpty(userFeature.getUser_cycle_bucket_30days()))
                .setUserShareBucket30Day(Strings.nullToEmpty(userFeature.getUser_share_bucket_30days()));

        if (userFeature.getDay1_cnt_features() != null) {
            builder.setDay1CntFeature(convert(userFeature.getDay1_cnt_features()));
        }
        if (userFeature.getDay3_cnt_features() != null) {
            builder.setDay3CntFeature(convert(userFeature.getDay3_cnt_features()));
        }
        if (userFeature.getDay7_cnt_features() != null) {
            builder.setDay7CntFeature(convert(userFeature.getDay7_cnt_features()));
        }
        if (userFeature.getMonth3_cnt_features() != null) {
            builder.setMonth3CntFeature(convert(userFeature.getMonth3_cnt_features()));
        }

        return builder.build();
    }

    private static UserActionFeatureProto convert(UserActionFeature feature) {
        if (feature == null) {
            return null;
        }
        UserActionFeatureProto.Builder builder = UserActionFeatureProto.newBuilder();
        builder.setCtr(feature.getCtr())
                .setStr(feature.getStr())
                .setRov(feature.getRov())
                .setShareCnt(Math.round(feature.getShare_cnt()))
                .setReturnCnt(Math.round(feature.getReturn_cnt()))
                .setClickCnt(Math.round(feature.getClick_cnt()))
                .setExpCnt(Math.round(feature.getExp_cnt()));

        return builder.build();
    }


    public static VideoFeatureProto convert(ItemFeature videoFeature) {
        if (videoFeature == null) {
            return null;
        }
        VideoFeatureProto.Builder builder = VideoFeatureProto.newBuilder();

        builder.setVideoId(Strings.nullToEmpty(videoFeature.getVideoId()))
                .setUpId(Strings.nullToEmpty(videoFeature.getUpId()))
                .setTags(Strings.nullToEmpty(videoFeature.getTags()))
                .setPlayLength(Strings.nullToEmpty(videoFeature.getPlayLength()))
                .setTotalTime(Strings.nullToEmpty(videoFeature.getTotalTime()))
                .setDaysSinceUpload(Strings.nullToEmpty(videoFeature.getDaysSinceUpload()));

        if (videoFeature.getDay1_cnt_features() != null) {
            builder.setVideoDay1CntFeature(convert(videoFeature.getDay1_cnt_features()));
        }
        if (videoFeature.getDay3_cnt_features() != null) {
            builder.setVideoDay3CntFeature(convert(videoFeature.getDay3_cnt_features()));
        }
        if (videoFeature.getDay7_cnt_features() != null) {
            builder.setVideoDay7CntFeature(convert(videoFeature.getDay7_cnt_features()));
        }
        if (videoFeature.getMonth3_cnt_features() != null) {
            builder.setVideoMonth3CntFeature(convert(videoFeature.getMonth3_cnt_features()));
        }

        return builder.build();
    }

    public static UserAdFeatureProto convertToProto(UserAdFeature userAdFeature) {
        UserAdFeatureProto.Builder builder = UserAdFeatureProto.newBuilder();
        if (userAdFeature == null) return builder.build();
        builder.setMid(userAdFeature.getMid());

        if (userAdFeature.getDay1_cnt_features() != null) {
            builder.setDay1CntFeatures(convertToProto(userAdFeature.getDay1_cnt_features()));
        }
        if (userAdFeature.getDay3_cnt_features() != null) {
            builder.setDay3CntFeatures(convertToProto(userAdFeature.getDay3_cnt_features()));
        }
        if (userAdFeature.getDay7_cnt_features() != null) {
            builder.setDay7CntFeatures(convertToProto(userAdFeature.getDay7_cnt_features()));
        }
        if (userAdFeature.getMonth3_cnt_features() != null) {
            builder.setMonth3CntFeatures(convertToProto(userAdFeature.getMonth3_cnt_features()));
        }
        return builder.build();
    }

    public static AdActionFeatureProto convertToProto(AdActionFeature adActionFeature) {

        AdActionFeatureProto.Builder builder = AdActionFeatureProto.newBuilder();
        if (adActionFeature == null) return builder.build();
        builder.setAdView(adActionFeature.getAdView());
        builder.setAdClick(adActionFeature.getAdClick());
        builder.setAdConversion(adActionFeature.getAdConversion());
        builder.setCtr(adActionFeature.getCtr());
        builder.setCvr(adActionFeature.getCvr());

        return builder.build();
    }

    public static AdItemFeatureProto convertToProto(AdItemFeature adItemFeature) {
        if (adItemFeature == null) return null;
        AdItemFeatureProto.Builder builder = AdItemFeatureProto.newBuilder();
        builder.setAdId(adItemFeature.getAdId());
        builder.setAdCode(adItemFeature.getAdCode());
        builder.setAdvertiserId(adItemFeature.getAdvertiserId());
        builder.setAdvertiserCode(adItemFeature.getAdvertiserCode());
        builder.setCampaignId(adItemFeature.getCampaignId());
        builder.setCampaignCode(adItemFeature.getCampaignCode());
        builder.setCreativeId(adItemFeature.getCreativeId());
        builder.setCreativeCode(adItemFeature.getCreativeCode());

        if (adItemFeature.getDay1_cnt_features() != null) {
            builder.setDay1CntFeatures(convertToProto(adItemFeature.getDay1_cnt_features()));
        }
        if (adItemFeature.getDay3_cnt_features() != null) {
            builder.setDay3CntFeatures(convertToProto(adItemFeature.getDay3_cnt_features()));
        }
        if (adItemFeature.getDay7_cnt_features() != null) {
            builder.setDay7CntFeatures(convertToProto(adItemFeature.getDay7_cnt_features()));
        }
        if (adItemFeature.getMonth3_cnt_features() != null) {
            builder.setMonth3CntFeatures(convertToProto(adItemFeature.getMonth3_cnt_features()));
        }
        if (adItemFeature.getCreative_1day_cnt_features() != null) {
            builder.setCreative1DayCntFeatures(convertToProto(adItemFeature.getCreative_1day_cnt_features()));
        }
        if (adItemFeature.getCreative_3day_cnt_features() != null) {
            builder.setCreative3DayCntFeatures(convertToProto(adItemFeature.getCreative_3day_cnt_features()));
        }
        if (adItemFeature.getCreative_7day_cnt_features() != null) {
            builder.setCreative7DayCntFeatures(convertToProto(adItemFeature.getCreative_7day_cnt_features()));
        }
        if (adItemFeature.getCreative_3month_cnt_features() != null) {
            builder.setCreative3MonthCntFeatures(convertToProto(adItemFeature.getCreative_3month_cnt_features()));
        }
        if (adItemFeature.getAdvertiser_1day_cnt_features() != null) {
            builder.setAdvertiser1DayCntFeatures(convertToProto(adItemFeature.getAdvertiser_1day_cnt_features()));
        }
        if (adItemFeature.getAdvertiser_3day_cnt_features() != null) {
            builder.setAdvertiser3DayCntFeatures(convertToProto(adItemFeature.getAdvertiser_3day_cnt_features()));
        }
        if (adItemFeature.getAdvertiser_7day_cnt_features() != null) {
            builder.setAdvertiser7DayCntFeatures(convertToProto(adItemFeature.getAdvertiser_7day_cnt_features()));
        }
        if (adItemFeature.getAdvertiser_3month_cnt_features() != null) {
            builder.setAdvertiser3MonthCntFeatures(convertToProto(adItemFeature.getAdvertiser_3month_cnt_features()));
        }

        return builder.build();
    }
}
