package com.tzld.piaoquan.recommend.feature.service;

import com.ctrip.framework.apollo.spring.annotation.ApolloJsonValue;
import com.google.common.base.Strings;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.FeatureKeyProto;
import com.tzld.piaoquan.recommend.feature.model.feature.MultiGetFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.MultiGetFeatureResponse;
import com.tzld.piaoquan.recommend.feature.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;

/**
 * @author dyp
 */
@Service
@Slf4j
public class FeatureV2Service {
    @Qualifier("redisTemplate")
    @Autowired
    private RedisTemplate<String, String> redisTemplate;

    @ApolloJsonValue("${dts.config.v2:}")
    private List<DTSConfig> dtsConfigs;

    public MultiGetFeatureResponse multiGetFeature(MultiGetFeatureRequest request) {
        if (request.getFeatureKeyCount() == 0) {
            return MultiGetFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        // 目前都在一个Redis，所以放在一个list简化处理
        List<String> redisKeys = CommonCollectionUtils.toList(request.getFeatureKeyList(), fk -> redisKey(fk));
        List<String> values = redisTemplate.opsForValue().multiGet(redisKeys);

        // log.info("feature key {} value {}", JSONUtils.toJson(redisKeys), JSONUtils.toJson(values));

        MultiGetFeatureResponse.Builder builder = MultiGetFeatureResponse.newBuilder();
        builder.setResult(Result.newBuilder().setCode(1));
        for (int i = 0; i < request.getFeatureKeyCount(); i++) {
            builder.putFeature(request.getFeatureKeyList().get(i).getUniqueKey(),
                    Strings.nullToEmpty(values.get(i)));
        }
        return builder.build();
    }

    // Note：写入和读取的key生成规则应保持一致
    private String redisKey(FeatureKeyProto fk) {
        Optional<DTSConfig> optional = dtsConfigs.stream()
                .filter(c -> c.getOdps() != null && StringUtils.equals(c.getOdps().getTable(), fk.getTableName()))
                .findFirst();
        if (!optional.isPresent()) {
            log.error("table {} not config", fk.getTableName());
            return "";
        }
        DTSConfig config = optional.get();

        // Note：写入和读取的key生成规则应保持一致
        List<String> fields = config.getRedis().getKey();
        if (org.apache.commons.collections4.CollectionUtils.isEmpty(fields)) {
            return config.getRedis().getPrefix();
        }
        StringBuilder sb = new StringBuilder();
        if (StringUtils.isNotBlank(config.getRedis().getPrefix())) {
            sb.append(config.getRedis().getPrefix());
        }
        for (String field : fields) {
            sb.append(":");
            sb.append(fk.getFieldValueMap().get(field));
        }
        return sb.toString();
    }

}
