package com.tzld.piaoquan.recommend.feature.service;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.AdItemFeature;
import com.tzld.piaoquan.recommend.feature.domain.ad.base.UserAdFeature;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.GetUserAdFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.GetUserAdFeatureResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

@Service
@Slf4j
public class UserAdFeatureService extends AbstractFeatureService<String, UserAdFeature> {
    public UserAdFeatureService() {
        super.emptyData = "{}";
        super.emptyDataExpire = 0;
        super.defaultExpire = 0;
        super.keyFunc = k -> String.format("user:ad:%s", k);
        super.typeToken = new TypeToken<UserAdFeature>() {
        };

        super.defaultValueFunc = k -> {
            UserAdFeature feature = new UserAdFeature();
            feature.setMid(k);
            return feature;
        };

        int maximumSize = 10000;
        int refreshAfterWrite = 60;
        int expireAfterWrite = 60;
        int expireAfterAccess = 60;
        initLocalCache(maximumSize, refreshAfterWrite, expireAfterWrite, expireAfterAccess);
    }

    public GetUserAdFeatureResponse getUserAdFeature(GetUserAdFeatureRequest request) {
        String uid = request.getMid();
        if (StringUtils.isBlank(uid)) {
            return GetUserAdFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        UserAdFeature userAdFeature = super.get(uid);
        GetUserAdFeatureResponse.Builder builder = GetUserAdFeatureResponse.newBuilder();
        builder.setResult(Result.newBuilder().setCode(1));
        if (userAdFeature != null) {
            builder.setUserAdFeature(FeatureConverter.convertToProto(userAdFeature));
        }
        return builder.build();
    }
}


