package com.tzld.piaoquan.recommend.feature.service;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.domain.video.base.UserFeature;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.GetUserFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.GetUserFeatureResponse;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.StringUtils;
import org.springframework.stereotype.Service;

/**
 * @author dyp
 */
@Service
@Slf4j
public class UserFeatureService extends AbstractFeatureService<String, UserFeature> {

    public UserFeatureService() {
        super.emptyData = "{}";
        super.emptyDataExpire = 0;
        super.defaultExpire = 0;
        super.keyFunc = k -> String.format("user:video:%s", k);
        super.typeToken = new TypeToken<UserFeature>() {
        };
        super.defaultValueFunc = k -> UserFeature.defaultInstance(k);

        int maximumSize = 10000;
        int refreshAfterWrite = 60;
        int expireAfterWrite = 60;
        int expireAfterAccess = 60;
        initLocalCache(maximumSize, refreshAfterWrite, expireAfterWrite, expireAfterAccess);
    }


    public GetUserFeatureResponse getUserFeature(GetUserFeatureRequest request) {
        String mid = request.getMid();
        if (StringUtils.isBlank(mid)) {
            return GetUserFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        UserFeature userFeature = super.get(mid);
        GetUserFeatureResponse.Builder builder = GetUserFeatureResponse.newBuilder();
        builder.setResult(Result.newBuilder().setCode(1));
        if (userFeature != null) {
            builder.setUserFeature(FeatureConverter.convert(userFeature));
        }
        return builder.build();
    }

//    public void saveUserFeature(String jsonValue) {
//
//
//        UserFeature userFeature = JSONUtils.fromJson(jsonValue, new TypeToken<UserFeature>() {
//        }, null);
//        super.save(userFeature.getUid(), userFeature);
//    }


}
