package com.tzld.piaoquan.recommend.feature.service;

import com.google.common.reflect.TypeToken;
import com.tzld.piaoquan.recommend.feature.domain.video.base.ItemFeature;
import com.tzld.piaoquan.recommend.feature.model.common.Result;
import com.tzld.piaoquan.recommend.feature.model.feature.GetAllVideoFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.GetAllVideoFeatureResponse;
import com.tzld.piaoquan.recommend.feature.model.feature.GetVideoFeatureRequest;
import com.tzld.piaoquan.recommend.feature.model.feature.GetVideoFeatureResponse;
import com.tzld.piaoquan.recommend.feature.util.CommonCollectionUtils;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.collections4.MapUtils;
import org.apache.commons.lang.StringUtils;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Map;

/**
 * @author dyp
 */
@Service
@Slf4j
public class VideoFeatureService extends AbstractFeatureService<String, ItemFeature> {

    public VideoFeatureService() {
        super.emptyData = "{}";
        super.emptyDataExpire = 0;
        super.defaultExpire = 0;
        super.keyFunc = k -> String.format("video:%s", k);
        super.typeToken = new TypeToken<ItemFeature>() {
        };
        super.defaultValueFunc = k -> ItemFeature.defaultInstance(k);

        int maximumSize = 10000;
        int refreshAfterWrite = 60;
        int expireAfterWrite = 60;
        int expireAfterAccess = 60;

        initLocalCache(maximumSize, refreshAfterWrite, expireAfterWrite, expireAfterAccess);
    }


    public GetVideoFeatureResponse getVideoFeature(GetVideoFeatureRequest request) {
        String videoId = request.getVideoId();
        if (StringUtils.isBlank(videoId)) {
            return GetVideoFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        ItemFeature videoFeature = super.get(videoId);
        GetVideoFeatureResponse.Builder builder = GetVideoFeatureResponse.newBuilder();
        builder.setResult(Result.newBuilder().setCode(1));
        if (videoFeature != null) {
            builder.setVideoFeature(FeatureConverter.convert(videoFeature));
        }
        return builder.build();
    }

//    public void saveVideoFeature(String jsonValue) {
//
//        VideoFeature videoFeature = JSONUtils.fromJson(jsonValue, new TypeToken<VideoFeature>() {
//        }, null);
//        super.save(videoFeature.getVideoId(), videoFeature);
//    }

    public GetAllVideoFeatureResponse getAllVideoFeature(GetAllVideoFeatureRequest request) {
        if (request.getVideoIdCount() == 0) {
            return GetAllVideoFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }
        List<String> videoIds = request.getVideoIdList();

        Map<String, ItemFeature> videoFeatureMap = super.getAll(videoIds);
        if (MapUtils.isEmpty(videoFeatureMap)) {
            return GetAllVideoFeatureResponse.newBuilder()
                    .setResult(Result.newBuilder().setCode(1))
                    .build();
        }

        return GetAllVideoFeatureResponse.newBuilder()
                .setResult(Result.newBuilder().setCode(1))
                .addAllVideoFeature(CommonCollectionUtils.toList(videoFeatureMap.values(), FeatureConverter::convert))
                .build();
    }
}
